--
-- HelperAdmin
-- V1.0.0.0
--
-- @author apuehri
-- @date 05/01/2018
--
-- Copyright (C) apuehri
-- V1.0.0.0 ..... first implementation
--          ..... 06/05/2018 integration gui

helperadmin = {};
local modItem = ModsUtil.findModItemByModName(g_currentModName);
helperadmin.Version = Utils.getNoNil(modItem.version, "0.0.0.0");
helperadmin.dir = g_currentModDirectory;
helperadmin.debug = false;
helperadmin.minHelper = 6;
helperadmin.maxHelper = 20;

function helperadmin.prerequisitesPresent(specializations)
    return true;
end;

function helperadmin:loadMap(name)
	--setTextGlobal
	setTextGlobalI18N()
	--initialize
	helperadmin.nameMan = {};
	helperadmin.nameWoman = {};
	helperadmin.newNumHirables = 6;
	
	--gui
    helperadmin.gui = {};
    helperadmin.gui["haSettingGui"] = haGui:new();
    g_gui:loadGui(helperadmin.dir .. "gui/haGui.xml", "haGui", helperadmin.gui.haSettingGui);
	
	--load Savegame
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;

	if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
		if fileExists(savegameFolderPath .. '/helperAdmin.xml') then
			print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by aPuehri --- loading savegame ---");
			local key = "helperAdmin";
			local xmlFile = loadXMLFile("helperAdmin", savegameFolderPath .. "/helperAdmin.xml", key);			
			if xmlFile ~= nil then
				for i=1, helperadmin.maxHelper do
					-- man
					local XmlNameMan = getXMLString(xmlFile, key .."#nameMan"..i);
					if XmlNameMan ~= nil then
						helperadmin.nameMan[i] = XmlNameMan;
					end;
					-- woman
					local XmlNameWoman = getXMLString(xmlFile, key .."#nameWoman"..i);
					if XmlNameWoman ~= nil then
						helperadmin.nameWoman[i] = XmlNameWoman;
					else
						break;						
					end;
				end;
			end;
			delete(xmlFile);
		else
			helperadmin.nameMan = {"Andy";"Phillip";"Franz";};
			helperadmin.nameWoman = {"Betty";"Sarah";"Elisabeth";};
			print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by aPuehri --- loading initialvalues ---");			
		end;		
	else
		helperadmin.nameMan = {"Andy";"Phillip";"Franz";};
		helperadmin.nameWoman = {"Betty";"Sarah";"Elisabeth";};
		print("--- loading HelperAdmin V".. helperadmin.Version .. " (c) by aPuehri --- loading initialvalues ---");	
	end;	
	
	-- register new helpers
	if (g_currentMission ~= nil) and (HelperUtil ~= nil) then
		self:RegNewHelpers();
	end;
end;

function helperadmin:deleteMap()
end;

function helperadmin:mouseEvent(posX, posY, isDown, isUp, button)
end;

function helperadmin:keyEvent(unicode, sym, modifier, isDown)
end;

function helperadmin:update(dt)
	if (#HelperUtil.availableHelpers == helperadmin.newNumHirables) then  
		if InputBinding.hasEvent(InputBinding.hat_togglehud) then
			if helperadmin.gui.haSettingGui.isOpen then
				helperadmin.gui.haSettingGui:onClickBack()
			elseif g_gui.currentGui == nil then
				g_gui:showGui("haGui")
			end
		end;
	end;
end;

function helperadmin:draw()   

end;

function helperadmin:LoadDataGui()   
	helperadmin.gui.haSettingGui:setNumHelper(g_currentMission.maxNumHirables);
	for i=1, (g_currentMission.maxNumHirables / 2) do
		helperadmin.gui.haSettingGui:setNameHelper(i, helperadmin.nameMan[i],1);
		helperadmin.gui.haSettingGui:setNameHelper(i, helperadmin.nameWoman[i],2);
	end;
end;

function helperadmin:UpdHelperFromGui(SelNumHelp, InputNameHelperM, InputNameHelperW)   
	if (SelNumHelp ~= nil) then
		helperadmin.nameMan = {};
		helperadmin.nameWoman = {};		
		-- get helper names from gui input
		for i = 1, SelNumHelp, 1 do
			if (InputNameHelperM[i] ~= nil) and (InputNameHelperM[i] ~= "") then
				helperadmin.nameMan[i] = InputNameHelperM[i];
			else
				helperadmin.nameMan[i] = "Man" ..i;
			end;
			if (InputNameHelperW[i] ~= nil) and (InputNameHelperW[i] ~= "") then
				helperadmin.nameWoman[i] = InputNameHelperW[i];
			else
				helperadmin.nameWoman[i] = "Woman" ..i;
			end;
		end	
		
		if helperadmin.debug then
			print("--- HelperAdmin Debug ... helperadmin:UpdHelperFromGui(indexNumHelp)= ".. tostring(SelNumHelp));
			print("--- HelperAdmin Debug ... helperadmin:UpdHelperFromGui(helperadmin.nameMan)");
			DebugUtil.printTableRecursively(helperadmin.nameMan,"*n* ", 0, 1);
			print("--- HelperAdmin Debug ... helperadmin:UpdHelperFromGui(helperadmin.nameWoman)");
			DebugUtil.printTableRecursively(helperadmin.nameWoman,"*n* ", 0, 1);
		end;
		
	end;
end;

function helperadmin:RegNewHelpers()
	if (#helperadmin.nameMan == #helperadmin.nameWoman) then
		helperadmin.newNumHirables = #helperadmin.nameMan + #helperadmin.nameWoman;
		if (helperadmin.newNumHirables ~= nil) and (helperadmin.newNumHirables >= helperadmin.minHelper) and (helperadmin.newNumHirables <= helperadmin.maxHelper) then
			g_currentMission.maxNumHirables	= helperadmin.newNumHirables;
			if helperadmin.debug then
				print("--- HelperAdmin Debug ... helperadmin:RegNewHelpers(g_currentMission.maxNumHirables) = ".. g_currentMission.maxNumHirables);
			end;
		else
			print("--- HelperAdmin Error ... Number of helpers too low (min " ..tostring(helperadmin.minHelper)") or too high (max" ..tostring(helperadmin.maxHelper)"), no changes have been made !! ---");
			return;
		end;

		HelperUtil.NUM_HELPERS = 0;
		HelperUtil.helpers = {};
		HelperUtil.helperIndexToDesc = {}; 
		HelperUtil.availableHelpers = {};	
		
		-- Register new Helpers
		-- Info helperxx.xml
		-- 		man (helper01.xml,helper02.xml,helper03.xml,helper05.xml,helper06.xml)
		-- 		woman (helper04.xml,helper07.xml,helper08.xml,helper09.xml,helper10.xml)
		local HelperManXml = {[1]="dataS2/character/helper/helper01.xml",[2]="dataS2/character/helper/helper02.xml",[3]="dataS2/character/helper/helper03.xml",[4]="dataS2/character/helper/helper05.xml",[5]="dataS2/character/helper/helper06.xml"};
		local HelperWomanXml = {[1]="dataS2/character/helper/helper04.xml",[2]="dataS2/character/helper/helper07.xml",[3]="dataS2/character/helper/helper08.xml",[4]="dataS2/character/helper/helper09.xml",[5]="dataS2/character/helper/helper10.xml"};	
		local manNameIndex = 1;
		local womanNameIndex = 1;
		for i=1, g_currentMission.maxNumHirables do
			local calcmod = math.fmod(i,2);
			local maxNameIndex = 0;
			if calcmod == 1 then				
				local xmlIndex = math.random(1,5);
				HelperUtil.registerHelper(helperadmin.nameMan[manNameIndex], HelperManXml[xmlIndex]);
				maxNameIndex = #helperadmin.nameMan;
				if manNameIndex < maxNameIndex then
					manNameIndex = manNameIndex + 1;
				else
					manNameIndex = 1;
				end;				
			else				
				local xmlIndex = math.random(1,5);
				HelperUtil.registerHelper(helperadmin.nameWoman[womanNameIndex], HelperWomanXml[xmlIndex]);
				maxNameIndex = #helperadmin.nameWoman;
				if womanNameIndex < maxNameIndex then
					womanNameIndex = womanNameIndex + 1;
				else
					womanNameIndex = 1;
				end;
			end;	
		end;
		-- Remove original Entries in HelperUtil
		for k, v in pairs(HelperUtil) do
			if "HELPER_A" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_B" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_C" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_D" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_E" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_F" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_G" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_H" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_I" == k then
				HelperUtil = self:tremove(HelperUtil, k);
			elseif "HELPER_J" == k then
				HelperUtil = self:tremove(HelperUtil, k);				
			end;
		end;
		if helperadmin.debug then
			print("--- HelperAdmin Debug --- helperadmin:RegNewHelpers List of new Helpers........");
			DebugUtil.printTableRecursively(HelperUtil,"*n* ", 0, 5);		
		end;

		-- update Gui-Data
		helperadmin:LoadDataGui();
	else
		print("--- HelperAdmin Error ... Number of male and female helpers is different, no changes have been made !! ---");
	end;
end;

function helperadmin:tremove(t, k_to_remove)
	-- Lua table remove by key
	-- Code snippet from https://swfoo.com/2014/07/11/lua-table-remove-by-key/
	local new = {};
	for k, v in pairs(t) do
		new[k] = v;
	end;
	new[k_to_remove] = nil;
	return new;
end;

function helperadmin:saveSettings()
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;
	
	if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
		local key = "helperAdmin";
		local xmlFile = createXMLFile("helperAdmin", savegameFolderPath .. "/helperAdmin.xml", key);
		
		for i=1, #helperadmin.nameMan do			
			setXMLString(xmlFile, key .."#nameMan"..i, helperadmin.nameMan[i]);
		end;
		for i=1, #helperadmin.nameWoman do
			setXMLString(xmlFile, key .."#nameWoman"..i, helperadmin.nameWoman[i]);
		end;			
		saveXMLFile(xmlFile);
		delete(xmlFile);	
	end;
end;

g_careerScreen.saveSavegame = Utils.appendedFunction(g_careerScreen.saveSavegame, helperadmin.saveSettings);

addModEventListener(helperadmin); 
