--
-- HelperAdmin-Tools
-- V1.0.0.0
--
-- @author apuehri
-- @date 08/01/2018
--
-- Copyright (C) apuehri
-- V1.0.0.0 ..... first implementation

helperadmintools = {};
helperadmintools.debug = false;

function helperadmintools.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Hirable, specializations) 
		 and SpecializationUtil.hasSpecialization(helperadmintools, specializations)
end;

function helperadmintools:load(saveGame)
	--initialize
	self.hatShowWorkWidth = false;
	self.hatDrawLine = false;
	self.hatKeyReleased = false;
	self.hatWorkWidth = 0;
	self.hatLeRiOffset = 0;
	local hatPrintDebugDone = false;
	
	--Node
	local tempNode = self.aiVehicleDirectionNode
	if self.aiVehicleDirectionNode == nil then
		tempNode = self.components[1].node
	end
	if self.articulatedAxis ~= nil and self.articulatedAxis.componentJoint ~= nil and self.articulatedAxis.anchorActor ~= nil and self.articulatedAxis.componentJoint.jointNode ~= nil then				
		tempNode = getParent( self.articulatedAxis.componentJoint.jointNode )
	end
	
	self.hatRefNode = createTransformGroup("hatNewRefNode");
	link(tempNode,self.hatRefNode);
end;

function helperadmintools:delete()
	self.hatShowWorkWidth = false;
	self.hatDrawLine = false;
end;

function helperadmintools:mouseEvent(posX, posY, isDown, isUp, button)
end;

function helperadmintools:keyEvent(unicode, sym, modifier, isDown)
end;

function helperadmintools:update(dt)
	if self:canStartAIVehicle() then		
		if self:getIsActive() and self.isEntered then
			if InputBinding.hasEvent(InputBinding.hat_showworkwidth) then
				if not self.hatShowWorkWidth then
					self.hatShowWorkWidth = true;
					self.hatKeyReleased = false;
					
					-- Calculate workWidth
					local xmin = 0;
					local xmax = 0;
					xmin,xmax = helperadmintools.xMinMaxAI(self,self,xmin,xmax);
					xmin,xmax = helperadmintools.xMinMaxAreas(self,self.workAreas,xmin,xmax);
					
					--node
					local nodeVehicle;
					local nodeTool = {};
					local co;
					local cj;
					local degree;
					
					for nIndex,oImplement in pairs(self.attachedImplements) do --parse implements
						if oImplement ~= nil and oImplement.object ~= nil then
							-- node Vehicle
							nodeVehicle = self.attacherJoints[oImplement.jointDescIndex].rootNode;
							-- node Tool
							nodeTool[1] = oImplement.object.attacherJoint.rootNode;
							co = helperadmintools.tableGetN(oImplement.object.components);
							cj = helperadmintools.tableGetN(oImplement.object.componentJoints);
							if helperadmintools.debug then
								print("--- HelperAdmin-Tools Debug ...co(components)=" ..co.. ", ...cj(componentJoints)=" ..cj);
							end;							
							if (co >= 2) and (cj >= 1) then
								nodeTool[2] = oImplement.object.components[1].node;
								nodeTool[3] = oImplement.object.components[2].node;
							end;	
							-- workWidth
							xmin,xmax = helperadmintools.xMinMaxAI(self,oImplement.object,xmin,xmax);
							xmin,xmax = helperadmintools.xMinMaxAreas(self,oImplement.object.workAreas,xmin,xmax);
							
							for nIndex2,oImplement2 in pairs(oImplement.object.attachedImplements) do --parse 2nd line implements
								xmin,xmax = helperadmintools.xMinMaxAI(self,oImplement2.object,xmin,xmax);
								xmin,xmax = helperadmintools.xMinMaxAreas(self,oImplement2.object.workAreas,xmin,xmax);
							end
						end
					end
										
					--get angle between vehicle and tool
					degree = helperadmintools.getRelativeYRotation(nodeVehicle, nodeTool[1]);
					if helperadmintools.debug then
						print("--- HelperAdmin-Tools Debug ... degree nodeVehicle<-->nodeTool[1]=" ..degree);
					end;
					if (math.abs(degree) <= 0.01) and (co >= 2) and (cj >= 1) then
						degree = helperadmintools.getRelativeYRotation(nodeTool[2], nodeTool[3]);
						if helperadmintools.debug then
							print("--- HelperAdmin-Tools Debug ... degree nodeTool[2]<-->nodeTool[3]=" ..degree);
						end;
					end;

					--set workWidth and Offset
					local workWidth = math.abs(xmax-xmin);
					if workWidth > 0.1 then
						self.hatWorkWidth = workWidth;
						self.hatLeRiOffset = (xmin+xmax)/2;
						if math.abs(self.hatLeRiOffset) < 0.1 then
							self.hatLeRiOffset = 0;
						elseif math.abs(degree) > 0.01 then
							self.hatLeRiOffset = 0;
						end
						if helperadmintools.debug then
							print("--- HelperAdmin-Tools Debug ... xmin = " ..xmin.. " - xmax = " ..xmax.. " - self.hatWorkWidth = " ..self.hatWorkWidth.. " - self.hatLeRiOffset = " ..self.hatLeRiOffset);
						end;			
						-- Draw WorkWidth
						self.hatDrawLine = true;
						hatPrintDebugDone = false;
					end
				elseif self.hatShowWorkWidth and self.hatKeyReleased then
					self.hatShowWorkWidth = false;
					self.hatKeyReleased = false;
					self.hatDrawLine = false;
					if helperadmintools.debug then
						print("--- HelperAdmin-Tools Debug ... self.hatShowWorkWidth = " ..tostring(self.hatShowWorkWidth));
					end;							
				end;
				if helperadmintools.debug then
					print("--- HelperAdmin-Tools Debug ...self:canStartAIVehicle() = " ..tostring(self:canStartAIVehicle()).. "...self:getIsActive() = " ..tostring(self:getIsActive()).. "...self.isEntered = " ..tostring(self.isEntered));
				end;				
			end;
		end;
	end;
	
	-- Don't show WorkWidth when Helper is working
	if self.isHired then		
		self.hatShowWorkWidth = false;
		self.hatDrawLine = false;
	end;
	
	self.hatKeyReleased = true;	
end;

function helperadmintools:draw()
	if self.hatDrawLine then
		local left =  (self.hatWorkWidth *  0.5) + self.hatLeRiOffset;
		local right = (self.hatWorkWidth * -0.5) + self.hatLeRiOffset;
	
		if self.hatRefNode then
			if helperadmintools.debug and not hatPrintDebugDone then
				print("--- HelperAdmin-Tools Debug ... Draw with self.hatRefNode");
				hatPrintDebugDone = true;
			end;
			
			local p1x, p1y, p1z = localToWorld(self.hatRefNode, left,  0.5, 0);
			local p2x, p2y, p2z = localToWorld(self.hatRefNode, right, 0.5, 0);
			local p3x, p3y, p3z = localToWorld(self.hatRefNode, right, 0.5, 0);
			local p4x, p4y, p4z = localToWorld(self.hatRefNode, left,  0.5, 0);

			local p5x, p5y, p5z = localToWorld(self.hatRefNode, left,  0.5, 4.0);
			local p6x, p6y, p6z = localToWorld(self.hatRefNode, left,  0.5, -4.0);

			local p7x, p7y, p7z = localToWorld(self.hatRefNode, right, 0.5, 4.0);
			local p8x, p8y, p8z = localToWorld(self.hatRefNode, right, 0.5, -4.0);
					
			--Draw WorkWidth (blue)
			drawDebugLine(p1x, p1y, p1z, 0, 0, 1, p2x, p2y, p2z, 0, 0, 1);
			drawDebugLine(p2x, p2y, p2z, 0, 0, 1, p3x, p3y, p3z, 0, 0, 1);
			drawDebugLine(p3x, p3y, p3z, 0, 0, 1, p4x, p4y, p4z, 0, 0, 1);
			drawDebugLine(p4x, p4y, p4z, 0, 0, 1, p1x, p1y, p1z, 0, 0, 1);

			--Draw Helpline left (green)
			drawDebugLine(p5x, p5y, p5z, 0, 1, 0, p6x, p6y, p6z, 0, 1, 0);
			drawDebugLine(p6x, p6y, p6z, 0, 1, 0, p5x, p5y, p5z, 0, 1, 0);

			--Draw Helpline right (green)
			drawDebugLine(p7x, p7y, p7z, 0, 1, 0, p8x, p8y, p8z, 0, 1, 0);
			drawDebugLine(p8x, p8y, p8z, 0, 1, 0, p7x, p7y, p7z, 0, 1, 0);			
		else
			if helperadmintools.debug and not hatPrintDebugDone then
				print("--- HelperAdmin-Tools Debug ... Draw with self.rootNode");
				hatPrintDebugDone = true;
			end;
			
			local lX, lY, lZ = localToWorld(self.rootNode, left,  1.6, -6);
			local rX, rY, rZ = localToWorld(self.rootNode, right, 1.6, -6);

			drawDebugPoint(lX, lY, lZ, 1, 1, 0, 1);
			drawDebugPoint(rX, rY, rZ, 1, 1, 0, 1);

			drawDebugLine(lX, lY, lZ, 0, 0, 1, rX, rY, rZ, 0, 0, 1);				
		end;
	end;
end;					

function helperadmintools.xMinMaxAI(self,object,xmin,xmax)	
	if object.aiLeftMarker ~= nil and object.aiRightMarker ~= nil then		
		local x1,y1,z1 = getWorldTranslation(object.aiLeftMarker);	
		local x2,y2,z2 = getWorldTranslation(object.aiRightMarker);				
		local lx1,ly1,lz1 = worldToLocal(self.hatRefNode,x1,y1,z1);
		local lx2,ly2,lz2 = worldToLocal(self.hatRefNode,x2,y2,z2);	
		
		if lx1 < xmin then
			xmin = lx1;
		end
		if lx1 > xmax then
			xmax = lx1;
		end
		if lx2 < xmin then
			xmin = lx2;
		end
		if lx2 > xmax then
			xmax = lx2;
		end
	end;
	return xmin, xmax;
end;

function helperadmintools.xMinMaxAreas(self,areas,xmin,xmax)
	if areas ~= nil then
		for _,cuttingArea in pairs(areas) do
			if self:getIsWorkAreaActive(cuttingArea) then
				local x1,y1,z1 = getWorldTranslation(cuttingArea.start);
				local x2,y2,z2 = getWorldTranslation(cuttingArea.width);
				local x3,y3,z3 = getWorldTranslation(cuttingArea.height);
				local lx1,ly1,lz1 = worldToLocal(self.hatRefNode,x1,y1,z1);
				local lx2,ly2,lz2 = worldToLocal(self.hatRefNode,x2,y2,z2);
				local lx3,ly3,lz3 = worldToLocal(self.hatRefNode,x3,y3,z3);
				
				if lx1 < xmin then
					xmin = lx1;
				end
				if lx1 > xmax then
					xmax = lx1;
				end
				if lx2 < xmin then
					xmin = lx2;
				end
				if lx2 > xmax then
					xmax = lx2;
				end
				if lx3 < xmin then
					xmin = lx3;
				end
				if lx3 > xmax then
					xmax = lx3;
				end;
			end;
		end;
	end;
	return xmin, xmax;
end;

function helperadmintools.getRelativeYRotation(root,node)
	if root == nil or node == nil then
		return 0
	end
	local x, y, z = worldDirectionToLocal(node, localDirectionToWorld(root, 0, 0, 1))
	local dot = z
	dot = dot / Utils.vector2Length(x, z)
	local angle = math.acos(dot)
	if x < 0 then
		angle = -angle
	end
	return angle
end


function helperadmintools.tableGetN( req )
	if type( req ) == "table" then
		return table.getn( req )
	end
	return 0
end

